<?php

namespace Drupal\stripe_gateway;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\ConfigFactoryInterface;
use Stripe\StripeClient;

/**
 *
 * Service Api.
 *
 */
class StripeServiceApi
{

  /**
   * Drupal\Core\Config\ConfigFactory definition.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  public function __construct(ConfigFactoryInterface $config_factory)
  {
    $this->config = $config_factory->get('stripe_gateway.settings');
  }

  /**
   *  Cancel subscription using the Stripe API server.
   *
   * @param integer $id
   *    id ppss_sale
   * @param string $reason
   *    cancellation reason
   *
   * @return string
   *    cancellation status message
   */
  public function cancelSubscription($id, $reason)
  {
    // Get active subscription
    $query = \Drupal::database()->select('ppss_sales', 's');
    $query->condition('id', $id);
    $query->condition('status', 1);
    $query->isNull('expire');
    $query->fields('s');
    $result = $query->execute()->fetchAssoc();
    if($result) {
      $data = [];
      //reason for cancellation
      $data['reason'] = $reason;
      try {
        $stripe = $this->getStripeClient();
        $stripe->subscriptions->cancel(
          $result['id_subscription'],
          [
            "cancellation_details" => [
              'feedback' => $reason
            ]
          ]
        );
        return t('Cancellation subscription will be applied during the day.');

      } catch (\Stripe\Exception\InvalidRequestException $e) {
        return 'An error has occurred: ' . $e->getError()->message;
      }
    } else {
      return t('Subscription is not active.');
    }
  }

  /**
   *  Process a queue of webhook notification event customer.subscription.deleted
   *
   *  @param array $id
   *    Unique identifier for the object.
   *
   */
  public function receiveWebhookCancel($entity_data)
  {
    // Get all payments
    $query = \Drupal::database()->select('ppss_sales', 's');
    $query->join('ppss_sales_details', 'sd', 's.id = sd.sid');
    $query->condition('id_subscription', $entity_data->data->object->subscription);
    $query->fields('s', ['id','uid', 'status', 'id_role', 'id_subscription']);
    $query->fields('sd',['id', 'created']);
    $query->orderBy('created', 'DESC');
    $results = $query->execute()->fetchAll();
    if (!empty($results)) {
      $subscription = $results[0]; // Get the last payment
      $user = \Drupal\user\Entity\User::load($subscription->uid); // Get subscription user
      // Subscription expiration date
      $expire = $entity_data->data->object->current_period_end;
      $today = date('d-m-Y');
    
      // Validate expiration date with current expiration date
      if (date('d-m-Y', $expire) == $today) {
        try {
          // Update ppss_sales table
          \Drupal::database()->update('ppss_sales')->fields([
            'status' => 0,
            'expire' => $expire, ])->condition('id_subscription', $id, '=')->execute();
          // Remove user role added by subscription purchased
          $user->removeRole($subscription->id_role);
          $user->save();
          // Send email
          \Drupal::service('ppss.api_service')->sendEmail($user->getEmail(),
            'mail_canceled_subject', null);
          // Save log message
          \Drupal::logger('stripe gateway')->info(t('User @user subscription @id_subscription
            has been canceled', array('@id_subscription' => $subscription->id_subscription,
            '@user' => $subscription->uid)));
        } catch (\Exception $e) {
          \Drupal::logger('stripe gateway')->error($e->getMessage());
        }
      } else {
        // Update ppss_sales table
        \Drupal::database()->update('ppss_sales')->fields([
          'expire' => $expire,])->condition('id_subscription', $id, '=')->execute();

        // Send email
        \Drupal::service('ppss.api_service')->sendEmail($user->getEmail(),
          'mail_scheduled_cancellation_subject', $expire);
        // Save log message
        \Drupal::logger('stripe gateway')->info(t('User @user subscription @id_subscription will be
          canceled on @expire', array('@id_subscription' => $subscription->id_subscription,
          '@user' => $subscription->uid, "@expire" => date('d/m/Y', $expire))));
      }
      // Unpublish all ads setting a new date in the future
      \Drupal::service('ppss.api_service')->unpublishNodes($subscription->uid, $expire);
      
    }
  }

  /**
   * Save payment recurrent
   *
   * @param array $entity_data
   *   Data receive from webhook notification.
   */
  public function paymentCompleted($entity_data) {
    //get data subscription
    $query = \Drupal::database()->select('ppss_sales', 's');
    $query->condition('id_subscription', $entity_data->data->object->subscription);
    $query->fields('s', ['id','uid','frequency', 'status', 'details']);
    $results = $query->execute()->fetchAll();
    $subscription = $results[0];
    
    try {
      // Get the number of payments
      $payment = \Drupal::database()->select('ppss_sales_details', 's')
                  ->condition('sid', $subscription->id)
                  ->condition('event_id', 0)
                  ->fields('s')
                  ->execute()->fetchAll();
      if(count($payment) == 1){
        // if it is the first payment
        // update event_id from webhook
        \Drupal::database()->update('ppss_sales_details')->fields([
          'event_id' => $entity_data->id,
        ])->condition('id', $payment[0]->id, '=')->execute();
      } else {
        // Insert a new recurring payment
        $query = \Drupal::database()->insert('ppss_sales_details');
        $query->fields(['sid', 'tax', 'price', 'total', 'created', 'event_id']);
        $query->values([
          $subscription->id,
          number_format($entity_data->data->object->tax / 100, 2, '.', ''),
          number_format($entity_data->data->object->subtotal_excluding_tax / 100, 2, '.', ''),
          number_format($entity_data->data->object->total / 100, 2, '.', ''),
          $entity_data->created,
          $entity_data->id
        ]);
        $query->execute();
        \Drupal::logger('stripe gateway')->info(t('Recurring payment has been made: ', 
          ['@subscription' => $entity_data->data->object->subscription]));
      }
    } catch (\Exception $e) {
      \Drupal::logger('stripe gateway')->error($e->getMessage());
    }
  }

  /**
   * Get a Stripe Client.
   *
   * @return \Stripe\StripeClient
   *   The StripeClient.
   */
  public function getStripeClient() {
    return new StripeClient($this->getApiKey());
  }

  /**
   * Get secret key.
   *
   * @return string
   *   Secret key.
   */
  public function getApiKey() {
    // See your keys here: https://dashboard.stripe.com/apikeys
    if ($this->config->get('sandbox_mode')) {
      return $this->config->get('secret_key_test');
    } else {
      return $this->config->get('secret_key_live');
    }
  }
}
